#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Kysrun application launcher (http://colabti.org/kysrun).
Copyright (C) 2008  Francois Beerten

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import sys
import proposers
from qt import *

def startArgument():
    v = sys.argv[1]
    apps = proposers.findApplications(v)
    print apps
    if len(apps) > 0:
        startApplication(apps[0])

class Kysrun(QWidget):
    def __init__(self, *args):
        QWidget.__init__(self, *args)
        self.setCaption("Kysrun")
        self.setMinimumSize(400, 250)
        self.setMaximumSize(400, 250)
        self.layout = QGridLayout(self, 5, 1, 5, 10)
        self.title = QLabel(self)
        self.title.setFont(QFont("Sans", 11))
        self.title.setText('Kysrun')
        self.title.setMaximumWidth(390)
        self.detail = QLabel(self)
        self.detail.setFont(QFont("Sans", 8))
        self.detail.setMaximumWidth(390)
        self.search = QLineEdit(self)
        self.display = QLabel(self)
        self.propositions = QListBox(self)
        self.propositions.setMinimumSize(150, 100)
        self.propositions.setFont(QFont("Sans", 9))
        self.author = QLabel(self)
        self.author.setText('(c) 2008 Francois Beerten')
        self.layout.addWidget(self.title, 0, 0, Qt.AlignCenter)
        self.layout.addWidget(self.detail, 1, 0)
        self.layout.addWidget(self.search, 2, 0)
        self.layout.addMultiCellWidget(self.propositions, 3, 4, 0, 0)
        self.layout.addMultiCellWidget(self.display, 3, 4, 0, 0)
        self.layout.addWidget(self.author, 5, 0)
        self.display.hide()
        self.search.setFocus()
        self.connect(self.search, SIGNAL("textChanged(const QString&)"),
                     self.updateSearch)
        self.connect(self.search, SIGNAL("returnPressed()"),
                     self.startSelectedProposition)
        self.connect(self.propositions, SIGNAL("selected(int)"),
                     self.startSelectedProposition)
        self.props = []
        self.proposer = proposers.Proposer()
        self.updateSearch()
    def moveSelection(self, amount):
        if len(self.props):
            p = self.propositions.currentItem()
            p = divmod(p + amount, len(self.props))[1]
            self.setCurrent(p)
    def setCurrent(self, p):
        self.propositions.setCurrentItem(p)
        current = self.props[p]
        self.title.setText(current.display())
        self.detail.setText(current.detail())
    def keyPressEvent(self, event):
        k = event.key()
        if k == Qt.Key_Escape:
            self.close()
        elif k == Qt.Key_Down:
            self.moveSelection(1)
        elif k == Qt.Key_Up:
            self.moveSelection(-1)
        else:
            event.ignore()
    def clear(self):
        self.propositions.clear()
        self.title.setText('Kysrun')
        self.detail.setText('')
        self.props = []
    def updateSearch(self):
        self.clear()
        searchtext = str(self.search.text())
        self.props = self.proposer.find(searchtext)
        for s in self.props:
            pretty = s.display()
            self.propositions.insertItem(pretty)
        if len(self.props):
            self.setCurrent(0)
    def startSelectedProposition(self):
        if len(self.props) > 0:
            selected = self.propositions.currentItem()
            self.search.hide()
            self.propositions.hide()
            self.display.setText('Running %s ...' % self.props[selected].display())
            self.display.show()
            chosen = self.props[selected]
            chosen.start()
            self.proposer.saveChosenAction(chosen)
            QTimer.singleShot(800, self.close)
        else:
            self.close()


def centerWindow(app, window):
    d = app.desktop()
    w = d.width()
    h = d.height()
    x = w/2 - window.width()/2
    y = h/2 - window.height()/2
    window.move(x, y)

def runGui():
    app = QApplication(sys.argv)
    app.connect(app, SIGNAL('lastWindowClosed()'), app,
                SLOT('quit()'))
    gui = Kysrun(None, "Kysrun", Qt.WStyle_Customize | Qt.WStyle_NoBorder|
                                 Qt.WStyle_StaysOnTop)
    app.setMainWidget(gui)
    centerWindow(app, gui)
    gui.show()
    app.exec_loop()


def main():
    proposers.setupOnFirstStart()

    if len(sys.argv)  > 1 and not sys.argv[0].startswith('-'):
        startArgument()
    else:
        runGui()

if __name__ == "__main__":
    main()
