#! /usr/bin/env python
# -*- coding: utf-8 -*-
"""
Kysrun application launcher keyboard shortcut installation.
Copyright (C) 2008  Francois Beerten

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

import os
import shutil

home = os.path.expanduser('~')
icewmdir = home + '/.icewm'
keyspath = icewmdir + '/keys'

def RestartIcewm():
    print('Restarting the window manager.')
    os.popen("killall -HUP icewm")
    os.popen("killall -HUP icewm")
    print "Done."

def PrepareLocalConfiguration():
    if not os.access(icewmdir, os.R_OK):
        os.mkdir(icewmdir)
    if not os.access(keyspath, os.F_OK):
        if os.access('/etc/X11/icewm/keys', os.F_OK):
            shutil.copy('/etc/X11/icewm/keys', icewmdir)
        else:
            return False
    return True

def AddIcewmShortcut():
    if not PrepareLocalConfiguration():
        return False
    print "Adding a keyboard shortcut for Kysrun."
    if 'kysrun.py' in file(keyspath).read():
        print 'There is already a Kysrun shortcut.'
        return
    f = file(keyspath, 'a')
    f.write('# Kysrun shortcut\n')
    f.write('key "Alt+Space" \t\t%s/kysrun/kysrun.py' % home)
    RestartIcewm()


def AddMenuEntry():
    os.popen('xdg-desktop-menu install --novendor %s/kysrun/Kysrun.desktop' %
               home)


def findFreeCommandSlot(config):
    import re
    slots = range(0, 10)
    matches = re.compile('as_command(\d) =').findall(config)
    for m in matches:
        slots.remove(int(m))
    return slots[0]

def AddCompizShortcut():
    compizconfig = home + '/.config/compiz/compizconfig/Default.ini'
    if not os.access(compizconfig, os.F_OK):
        return
    print "Adding a keyboard shortcut for Kysrun in Compiz."
    config = file(compizconfig).read()
    if 'kysrun.py' in config:
        print 'There is already a Kysrun shortcut for Compiz.'
        return
    slot = int(findFreeCommandSlot(config))
    w = file(compizconfig, 'w')
    h, t = config.split('[core]', 1)
    w.write(h)
    w.write('[core]\n')
    w.write('as_run_command%d = <Alt>space,Button0,,0,false\n' % slot)
    w.write('as_command%d = %s/kysrun/kysrun.py\n' % (slot,home))
    w.write(t)
    w.close()
    # No need to restart compiz.real.
    # Compiz reads the new settings immediatly.
    


AddMenuEntry()
AddIcewmShortcut()
AddCompizShortcut()


print "Press ALT and SPACE to pop up kysrun."
